#!/usr/bin/env python3
#
#

import os
import sys

if sys.version_info.major < 3: print("you need to run zbot with python3") ; os._exit(1)

try: use_setuptools()
except: pass

try:
    from setuptools import setup
except Exception as ex:
    print(str(ex))
    os._exit(1)

from distutils import sysconfig
site_packages_path = sysconfig.get_python_lib()

setup(
    name='zbot',
    version='51',
    url='https://pikacode.com/milla/zbot',
    author='Bart Thate',
    author_email='milla@dds.nl',
    description='uses zbot.workdir in the current working directory, use -d <directory> if you need to access other directories.',
    license='MIT',
    include_package_data=True,
    zip_safe=False,
    install_requires=["distribute", "beautifulsoup4", "sleekxmpp", "natural", "feedparser"],
    scripts=["bin/zbot", "bin/zout", "bin/zbot-sed", ],
    packages=['zbot', 
              'zbot.plugs',
              'zbot.drivers',
             ],
    long_description = """ xmpp, irc, shell, cli, json - if you need to log effects of medicine, wait for version 100, email at milla@dds.nl

documentation is at http://pythonhosted.org/zbot/

:: 

  milla@reppa:~/zbot.dev/first$ ./bin/zbot -s -l warn

  ZBOT #44 -=- ! Thu Jan 23 16:28:55 2014

  16:28:55 -=- ! E G G S
  16:28:55 -=- !  
  16:28:55 -=- ! zbot egg: /usr/local/lib/python3.3/dist-packages/zbot-43-py3.3.egg
  16:28:55 -=- !  
  16:28:55 -=- ! C O N F I G
  16:28:55 -=- !  
  16:28:55 -=- ! loglevel=warn
  16:28:55 -=- ! workdir=/home/milla/zbot.dev/first/zbot.workdir
  16:28:55 -=- ! do_shell=True
  16:28:55 -=- ! rootlist=['/home/milla/zbot.dev/first/zbot.workdir', 
                           'Public/zbot.workdir/2014-01-23', 
                           'Dropbox/backups/zbot.workdir/2014-01-23']
  16:28:55 -=- !  
  16:28:55 -=- ! B O O T
  16:28:55 -=- !  
  16:28:55 -=- ! plugs /home/milla/zbot.dev/first/zbot/plugs
  16:28:55 -=- < 

  milla@reppa:~/zbot.dev/first$ ./bin/zbot dump log
  {"added": "Thu Jan 23 16:26:26 2014", "date": "Thu Jan 23 16:26:26 2014", "log": "dizzy"}
  {"added": "Thu Jan 23 16:26:07 2014", "date": "Thu Jan 23 16:26:07 2014", "log": "orap"}

Route

1.      Vertrek in noordwestelijke richting op de s112  3,7 km
2.      Neem de afslag links naar A10 richting Amersfoort/Zaanstad/s113/Ring 13,5 km
3.      Houd rechts aan bij het knooppunt Knooppunt Coenplein en volg de borden A8/E22 richting s118/Zaanstad/Purmerend/Leeuwarden      350 m
4.      Neem afslag Zaanstad links en voeg in op de A8/E22 richting Purmerend/Leeuwarden Ga verder op de A8     9,5 km
5.      Neem afslag Wormer naar N8/N246 richting Alkmaar/Beverwijk      950m
6.      Sla rechtsaf naar de Provincialeweg/N246/N8 Ga verder op de Provincialeweg/N246 9,3 km
7.      Sla linksaf naar de Zuiddijk/N244 Ga verder op de N244  2,8 km
8.      Flauwe bocht naar rechts naar de Westdijk/N244  6,7 km
9.      Flauwe bocht naar links om op de Westdijk/N244 te blijven       75 m
10.     Voeg in op de N242 via de oprit richting Ring Alkmaar (N.)/Schagen/Heerhugowaard        4,3 km
11.     Sla rechtsaf richting Westtangent       350 m
12.     Sla linksaf naar de Westtangent 2,6 km
13.     Sla rechtsaf naar de Icaruslaan 230 m
14.     Weg vervolgen naar Vondellaan   280 m
15.     Sla linksaf naar de Basiusstraat        210 m
16.     Neem de 3e afslag rechts, Raadhuisplein op      48 m
17.     Neem de 1e afslag links om op Raadhuisplein te blijven U vindt uw bestemming rechts


""",
    data_files=[('doc', ["LICENSE", ]),],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Topic :: Communications :: Email',
        'Topic :: Software Development :: Libraries :: Python Modules'],
)
