# zbot/drivers/irc.py
#
#

""" IRC bot. """

## IMPORTS

from zbot import Bot, Object, RemoteDisconnect, TargetMissing, kernel, handle_loop
from zbot.utils import error, split_txt, dispatch, resolve

import collections
import threading
import logging
import _thread
import socket
import time
import sys
import os
import re

## irc_parse function

def irc_parse(obj, *args, **kwargs):
    rawstr = str(args[0])
    zelf = obj
    zelf.servermsg = False
    splitted = re.split('\s+', rawstr)
    if not rawstr[0] == ':': zelf.servermsg = True
    zelf.prefix = splitted[0]
    if zelf.servermsg: zelf.etype = zelf.prefix
    else: zelf.etype = splitted[1]
    try: 
        nickuser = zelf.prefix.split('!')
        zelf.origin = nickuser[1]
        zelf.nick = nickuser[0][1:]
    except IndexError: zelf.origin = zelf.prefix ; zelf.servermsg = True
    if zelf.etype in pfc:
        zelf.arguments = splitted[2:pfc[zelf.etype]+2]
        txtsplit = re.split('\s+', rawstr, pfc[zelf.etype]+2)
        zelf.txt = txtsplit[-1]
    else: zelf.arguments = splitted[2:]
    if zelf.arguments: zelf.target = zelf.arguments[0]
    zelf.postfix = ' '.join(zelf.arguments)
    if not "txt" in zelf: zelf.txt = rawstr.rsplit(":")[-1]
    if zelf.txt.startswith(":"): zelf.txt = zelf.txt[1:]
    if not "channel" in zelf:
        for c in zelf.arguments + [zelf.txt, ]:
            if c.startswith("#"): zelf.channel = c
    if zelf.servermsg: 
        zelf.origin = zelf.origin[1:-1]
        zelf.channel = zelf.origin 
    if not "origin" in zelf: zelf.origin = zelf.channel
    if not "target" in zelf: zelf.target = zelf.channel or zelf.origin
    return obj

## IRCBot

class IRCBot(Bot):

    marker = "\r\n"
    cc = "."

    def __init__(zelf, *args, **kwargs):
        Bot.__init__(zelf, *args, **kwargs)
        zelf.connected = threading.Event()
        zelf.register("250", zelf._onconnect)
        zelf.register("001", zelf._onconnect)
        zelf.register("513", zelf.handle_513)
        zelf.register("433", zelf.handle_433)
        zelf.register("PING", zelf.handle_ping)
        zelf.register("INVITE", zelf.handle_invite)
        zelf.register("PRIVMSG", zelf.handle_privmsg)
        zelf._lock = _thread.allocate_lock()
        zelf._buffer = []
        zelf._lastline = ""
        zelf.encoding = "utf-8"
        zelf.server = zelf.server or "localhost"
        zelf.port = zelf.port or 6667
        zelf.nick = zelf.nick or "zbot"
        zelf.name = zelf.name or zelf.etype
        if not zelf.channels: zelf.channels = []
        if zelf.channel and zelf.channel not in zelf.channels: zelf.channels.append(zelf.channel)

    def _raw(zelf, txt):
        logging.warn("> %s" % txt)
        if not txt.endswith(zelf.marker): txt += zelf.marker
        itxt = bytes(txt, zelf.encoding)
        if zelf.outer: o = zelf.outer ; o.write(itxt) ; o.write("\n") ; o.flush() ; return		
        if 'ssl' in zelf and zelf.ssl: zelf.sock.write(itxt)
        else: zelf.sock.send(itxt[:512])

    def _connect(zelf):
        zelf.stopped = False
        if zelf.ipv6: zelf.oldsock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        else: zelf.oldsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        zelf.server = zelf.bind()
        logging.warn('connect %s - %s (%s)' % (zelf.server, zelf.port, zelf.name))
        zelf.oldsock.settimeout(60)
        zelf.oldsock.connect((zelf.server, int(str(zelf.port or 6667))))	
        zelf.blocking = 1
        zelf.oldsock.setblocking(zelf.blocking)
        logging.warn('connection made to %s (%s)' % (zelf.server, zelf.name))
        zelf.fsock = zelf.oldsock.makefile("r")
        if zelf.blocking:
            socktimeout = 301.0
            zelf.oldsock.settimeout(socktimeout)
        if 'ssl' in zelf and zelf['ssl']: zelf.sock = socket.ssl(zelf.oldsock) 
        else: zelf.sock = zelf.oldsock
        zelf.connecttime = time.time()
        return True

    def _onconnect(zelf, *args, **kwargs):
        logging.warn("logged on !!")
        if "onconnect" in zelf: time.sleep(2) ; zelf._raw(zelf.onconnect)
        if "servermodes" in zelf: zelf._raw("MODE %s %s" % (zelf.nick, zelf.servermodes))
        zelf.join_channels(*args, **kwargs)
        zelf.ready()

    def _dodcc(zelf, event, s):
        s.send(bytes('Welcome to ZBOT ' + event.nick + " !!\n", zelf.encoding))
        _thread.start_new_thread(zelf._dccloop, (event, s))

    def _dccloop(zelf, event, s):
        sockfile = s.makefile('rw')
        s.setblocking(True)
        while 1:
            try:
                res = sockfile.readline()
                if not res: break
                res = res.rstrip()
                logging.warn("< DCC %s" % res)
                o = Object(_target=zelf, txt=res)
                o.outer = sockfile
                zelf.put(o)
            except socket.timeout: time.sleep(0.01)
            except socket.error as ex:
                if ex.errno in [EAGAIN, ]: continue
                else: raise
            except Exception as ex: error()
        sockfile.close()

    def _dccconnect(zelf, event):
        try:
            addr = event.args[3] ; port = event.args[4][:-1]
            port = int(port)
            if re.search(':', addr): s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
            else: s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((addr, port))
        except Exception as ex: error() ; return
        zelf._dodcc(event, s)

    def get_one(zelf, *args, **kwargs):
        if not zelf._buffer: zelf.read_some()
        line = zelf._buffer.pop(0)
        e = Object()
        e._target = zelf
        o = irc_parse(e, line.rstrip())
        return o

    def read_some(zelf, *args, **kwargs):
        try:
            if "ssl" in zelf and zelf.ssl: inbytes = zelf.sock.read()
            else: inbytes = zelf.sock.recv(512)
        except Exception as ex: error()
        txt = str(inbytes, zelf.encoding)
        if txt == "": raise RemoteDisconnect()
        zelf._lastline += txt
        splitted = zelf._lastline.split(zelf.marker)
        for s in splitted[:-1]:
            logging.warn("< %s" % s)
            zelf._buffer.append(s)
        zelf._lastline = splitted[-1]

    def send(zelf, txt): zelf._raw(txt)

    def bind(zelf):
        server = zelf.server
        try: zelf.oldsock.bind((server, 0))
        except socket.error:
            if not server:
                try: socket.inet_pton(socket.AF_INET6, zelf.server)
                except socket.error: pass
                else: server = zelf.server
            if not server:  
                try: socket.inet_pton(socket.AF_INET, zelf.server)
                except socket.error: pass
                else: server = zelf.server
            if not server:
                ips = []
                try:
                    for item in socket.getaddrinfo(zelf.server, None):
                        if item[0] in [socket.AF_INET, socket.AF_INET6] and item[1] == socket.SOCK_STREAM:
                            ip = item[4][0]
                            if ip not in ips: ips.append(ip)
                except socket.error: pass
                else: server = random.choice(ips)
        return server

    def logon(zelf):
        if "password" in zelf: zelf._raw("PASS %s" % zelf.password)
        logging.warn('logging in on %s - this may take a while' % zelf.server)
        zelf._raw("NICK %s" % zelf.nick or "zbot")
        zelf._raw("USER %s localhost %s :%s" % (zelf.username or "zbot", zelf.server or "localhost", zelf.realname or "zbot"))

    def say(zelf, channel, txt):
        for text in txt.split("\n"):
            for line in split_txt(text):
                if line: zelf.privmsg(channel, line) 

    def join_channels(zelf, *args, **kwargs):
        for channel in zelf.channels: zelf.join(channel)

    def broadcast(zelf, txt):
        for i in zelf.channels: zelf.say(i, txt)

    def connect(zelf, reconnect=True):
        res = zelf._connect()
        if res: time.sleep(1) ; zelf.logon()
        return res

    def close(zelf):
        if 'ssl' in zelf and zelf['ssl']: zelf.oldsock.shutdown(2)
        else: zelf.sock.shutdown(2)
        if 'ssl' in zelf and zelf['ssl']: zelf.oldsock.close()
        else: zelf.sock.close()
        zelf.fsock.close()

    def handle_ping(zelf, event):
        zelf.pongcheck = True
        zelf.pong()

    def handle_433(zelf, event):  zelf.donick(event.arguments[1] + "_")

    def handle_invite(zelf, event): zelf.join(event.channel)

    def handle_privmsg(zelf, event):
        if event.txt.startswith("\001DCC"): zelf._dccconnect(event) ; return
        if event.txt and event.txt[0] == zelf.cc: event.is_cmnd = True ; resolve(event)

    def handle_513(zelf, event): zelf._raw("PONG %s" % event.arguments[6])

    def handle_ctcp(zelf, event):
        if event.txt and event.txt[0] == zelf.cc: resolve(event)

    def donick(zelf, nick, setorig=False, save=False, whois=False):
        nick = nick[:16]
        zelf.send('NICK %s\n' % nick)

    def join(zelf, channel, password=None):
        if password: zelf._raw('JOIN %s %s' % (channel, password))
        else: zelf._raw('JOIN %s' % channel)

    def part(zelf, channel):
        zelf.send('PART %s' % channel)
        if channel in zelf.channels: zelf.channels.remove(channel) ; zelf.save()

    def who(zelf, who): zelf.send('WHO %s' % who.strip())

    def names(zelf, channel): zelf.send('NAMES %s' % channel)

    def whois(zelf, who): zelf.send('WHOIS %s' % who)

    def privmsg(zelf, printto, what): zelf.send('PRIVMSG %s :%s' % (printto, what))

    def voice(zelf, channel, who): zelf.send('MODE %s +v %s' % (channel, who))
 
    def doop(zelf, channel, who): zelf.send('MODE %s +o %s' % (channel, who))

    def delop(zelf, channel, who): zelf.send('MODE %s -o %s' % (channel, who))

    def quit(zelf, reason='https://pikacode.com/milla/zbot'): zelf.send('QUIT :%s' % reason)

    def notice(zelf, printto, what): zelf.send('NOTICE %s :%s' % (printto, what))
 
    def ctcp(zelf, printto, what): zelf.send("PRIVMSG %s :\001%s\001" % (printto, what))

    def ctcpreply(zelf, printto, what): zelf.send("NOTICE %s :\001%s\001" % (printto, what))

    def action(zelf, printto, what, event=None, *args, **kwargs): zelf.send("PRIVMSG %s :\001ACTION %s\001" % (printto, what))

    def getchannelmode(zelf, channel): zelf.send('MODE %s' % channel)

    def settopic(zelf, channel, txt): zelf.send('TOPIC %s :%s' % (channel, txt))

    def ping(zelf, *args, **kwargs): zelf.send('PING :%s' % zelf.server)

    def pong(zelf, *args, **kwargs): zelf.send('PONG :%s' % zelf.server)

    def broadcast(zelf, txt):
        for i in zelf.channels: zelf.say(i, txt)

## POSTFIXCOUNT - how many arguments used in a IRC message

pfc = {}
pfc['NICK'] = 0
pfc['QUIT'] = 0
pfc['SQUIT'] = 1
pfc['JOIN'] = 0
pfc['PART'] = 1
pfc['TOPIC'] = 1
pfc['KICK'] = 2
pfc['PRIVMSG'] = 1
pfc['NOTICE'] = 1
pfc['SQUERY'] = 1
pfc['PING'] = 0
pfc['ERROR'] = 0
pfc['AWAY'] = 0
pfc['WALLOPS'] = 0
pfc['INVITE'] = 1
pfc['001'] = 1
pfc['002'] = 1
pfc['003'] = 1
pfc['004'] = 4
pfc['005'] = 15
pfc['302'] = 1
pfc['303'] = 1
pfc['301'] = 2
pfc['305'] = 1
pfc['306'] = 1
pfc['311'] = 5
pfc['312'] = 3
pfc['313'] = 2
pfc['317'] = 3
pfc['318'] = 2
pfc['319'] = 2
pfc['314'] = 5
pfc['369'] = 2
pfc['322'] = 3
pfc['323'] = 1
pfc['325'] = 3
pfc['324'] = 4
pfc['331'] = 2
pfc['332'] = 2
pfc['341'] = 3
pfc['342'] = 2
pfc['346'] = 3
pfc['347'] = 2
pfc['348'] = 3
pfc['349'] = 2
pfc['351'] = 3
pfc['352'] = 7
pfc['315'] = 2
pfc['353'] = 3
pfc['366'] = 2
pfc['364'] = 3
pfc['365'] = 2
pfc['367'] = 2
pfc['368'] = 2
pfc['371'] = 1
pfc['374'] = 1
pfc['375'] = 1
pfc['372'] = 1
pfc['376'] = 1
pfc['378'] = 2
pfc['381'] = 1
pfc['382'] = 2
pfc['383'] = 5
pfc['391'] = 2
pfc['392'] = 1
pfc['393'] = 1
pfc['394'] = 1
pfc['395'] = 1
pfc['262'] = 3
pfc['242'] = 1
pfc['235'] = 3
pfc['250'] = 1
pfc['251'] = 1
pfc['252'] = 2
pfc['253'] = 2
pfc['254'] = 2
pfc['255'] = 1
pfc['256'] = 2
pfc['257'] = 1
pfc['258'] = 1
pfc['259'] = 1
pfc['263'] = 2
pfc['265'] = 1
pfc['266'] = 1
pfc['401'] = 2
pfc['402'] = 2
pfc['403'] = 2
pfc['404'] = 2
pfc['405'] = 2
pfc['406'] = 2
pfc['407'] = 2
pfc['408'] = 2
pfc['409'] = 1
pfc['411'] = 1
pfc['412'] = 1
pfc['413'] = 2
pfc['414'] = 2
pfc['415'] = 2
pfc['421'] = 2
pfc['422'] = 1
pfc['423'] = 2
pfc['424'] = 1
pfc['431'] = 1
pfc['432'] = 2
pfc['433'] = 2
pfc['436'] = 2
pfc['437'] = 2
pfc['441'] = 3
pfc['442'] = 2
pfc['443'] = 3
pfc['444'] = 2
pfc['445'] = 1
pfc['446'] = 1
pfc['451'] = 1
pfc['461'] = 2
pfc['462'] = 1
pfc['463'] = 1
pfc['464'] = 1
pfc['465'] = 1
pfc['467'] = 2
pfc['471'] = 2
pfc['472'] = 2
pfc['473'] = 2
pfc['474'] = 2
pfc['475'] = 2
pfc['476'] = 2
pfc['477'] = 2
pfc['478'] = 3
pfc['481'] = 1
pfc['482'] = 2
pfc['483'] = 1
pfc['484'] = 1
pfc['485'] = 1
pfc['491'] = 1
pfc['501'] = 1
pfc['502'] = 1
pfc['700'] = 2
